#include "PSEmu PlugIn Defs.h"
                            
////////////////////////////////////////////////////////////////////////
// PPDK developer must change libraryName field and can change revision and build
////////////////////////////////////////////////////////////////////////

const  unsigned char version  = 1;    // do not touch - library for PSEmu 1.x
const  unsigned char revision = 1;
const  unsigned char build    = 0;
static char *libraryName      = "XYZ cdr Driver";
static char *PluginAuthor     = "XYZ";

////////////////////////////////////////////////////////////////////////
// stuff to make this a true PDK module
////////////////////////////////////////////////////////////////////////

char * CALLBACK PSEgetLibName(void)
{
 return libraryName;
}

unsigned long CALLBACK PSEgetLibType(void)
{
 return  PSE_LT_CDR;
}

unsigned long CALLBACK PSEgetLibVersion(void)
{
 return version<<16|revision<<8|build;
}

////////////////////////////////////////////////////////////////////////
// Init/shutdown, will be called just once on emu start/close
////////////////////////////////////////////////////////////////////////

long CALLBACK CDRinit(void)
{
 return 0;
}

long CALLBACK CDRshutdown(void)
{
 return 0;
}

////////////////////////////////////////////////////////////////////////
// Open/close should be called when a game starts/stops...
// well, some emus are using the funcs very lazily... if you do
// init/cleanup in the open/close funcs, be sure to add some
// checks in the other funcs as well, if your open has been
// already called
////////////////////////////////////////////////////////////////////////

long CALLBACK CDRopen(void)
{
 return 0;
}

long CALLBACK CDRclose(void)
{
 return 0;
}

////////////////////////////////////////////////////////////////////////
// Gets the first/last tracknumber
////////////////////////////////////////////////////////////////////////

long CALLBACK CDRgetTN(unsigned char *buffer)
{
 if(somethingisbad) return -1;

 buffer[0]=sTOC.cFirstTrack;
 buffer[1]=sTOC.cLastTrack;

 return 0;
}

////////////////////////////////////////////////////////////////////////
// Gets the track start pos in MSF time format
////////////////////////////////////////////////////////////////////////

long CALLBACK CDRgetTD(unsigned char track, unsigned char *buffer)
{
 // if track==0 -> return total length of cd
 // otherwise return start in bcd time format

 buffer[0]=mm;
 buffer[1]=ss;
 buffer[2]=ff;

 return 0;
}

////////////////////////////////////////////////////////////////////////
// Reads data into the internal cdr buffer
// the buffer should be 2340 bytes (without the
// 12 bytes sync data... so the real buffer is
// usually 2352 bytes, but the getBuffer func has
// to return a pointer to the real 2340 sector data)
////////////////////////////////////////////////////////////////////////

long CALLBACK CDRreadTrack(unsigned char *time)
{
 // read one sector at the given time address

 return 0;
}

////////////////////////////////////////////////////////////////////////
// Gets a pointer to the last readTrack sector data...
// an emu has to call readTrack first, and gets the
// sector buffer pointer after that, using getBuffer
////////////////////////////////////////////////////////////////////////

unsigned char * CALLBACK CDRgetBuffer(void)
{
 return PointerToLastReadTrackSectorMemory;
}

////////////////////////////////////////////////////////////////////////
// Various dlgs/test funcs
////////////////////////////////////////////////////////////////////////

long CALLBACK CDRconfigure(void)
{
 return 0;
}

void CALLBACK CDRabout(void)
{
}

long CALLBACK CDRtest(void)
{
 return 0;
}

////////////////////////////////////////////////////////////////////////
// CDDA funcs (optionally... only supported by PCSX right now)
////////////////////////////////////////////////////////////////////////

long CALLBACK CDRplay(unsigned char * sector)
{
 // starts playing an audio track at the given time address.
 // Unlike CDRreadTrack the address IS NOT BCD coded!
 
 return 0;
}

long CALLBACK CDRstop(void)
{
 // stops the cdda music... remember to do
 // an additional stop in CDRclose, or music
 // will play on when the main emu quits

 return 0;
}

////////////////////////////////////////////////////////////////////////
// SUBCHANNEL func
////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////
// Gets a pointer the 96 byte subchannel data buffer.
// an emu has to call readTrack first, and gets the
// sector buffer pointer after that, using getBufferSub
//
// the 96 bytes are like the one in a CloneCD SUB file, but
// my cdr plugin will only fill the sub q infos, so the buffer
// will look like:
// byte 0  - 11: 0
// byte 12 - 22: sub q data
// byte 23 - 95: 0
//
// the sub q data is build like that:
//
//  UCHAR  ControlAndADR;           (eg 0x41)
//  UCHAR  TrackNumber;             (eg 0x01)
//  UCHAR  IndexNumber;             (eg 0x01)
//  UCHAR  TrackRelativeAddress[3]; (bcd)
//  UCHAR  Filler;                  (always 0)
//  UCHAR  AbsoluteAddress[3];      (bcd)
//
// mmm... what else? my plugin will only return the exact
// subchannel data on DATA tracks... on AUDIO tracks the
// info is also given (the same 10 bytes will be filled), 
// so the emu can start audio play, and ask for the current
// position by calling "CDRgetBufferSub" without any "readTrack"
// calls, but the returned sub channel addresses will likely 
// be not exact (scsi command 0x42 used while audio playing)
//
// Another point: if the user has disabled subchannel reading 
// in the plugin config, the function will return NULL!
//
// And still more infos:
// My plugin can use "SBI" files. A "SBI" file has the following 
// structure:
//
// 4 bytes header (always "SBI\0")
// 3 bytes real absolute sector address where the sub q data was bad
// 1 byte Format: the format can be 1, 2 or 3:
// Format 1: complete 10 bytes sub q data
// Format 2: 3 bytes wrong relative address
// Format 3: 3 bytes wrong absolute address
//
// That's done to reduce the file size... if only rel/abs address
// is bad in sub q data, the record will only take 7 bytes. If both
// addresses are wrong, or the track/index is bad, a record of 14 bytes
// is used.
//
// And yet some more infos:
// The ePSXe M3S format is easier, it's just 16 bytes sub q data
// (10 bytes as above, and 6 additional (unneeded) sub q data...
// The M3S file will contain all sector infos from abs position 
// 3-0-0 to 3-59-74, no matter if the sub q data is fine or bad.
// 
////////////////////////////////////////////////////////////////////////

unsigned char * CALLBACK CDRgetBufferSub(void)
{
 return 0;
}

////////////////////////////////////////////////////////////////////////
// Drive letter func
//
// that is only supported in W2K/XP by my plugin right now
// (until I find a way to get the logical letter in W9X/ME)
// if the plugin doesn't know the letter, it will return 0
// oh, and CDRopen has to be called before that function will
// work (that's true for all functions but the init one, btw)
//
////////////////////////////////////////////////////////////////////////

char CALLBACK CDRgetDriveLetter(void)
{
 return 0;
}

////////////////////////////////////////////////////////////////////////
// Status func (only for pcsx right now)
//
// reads cdr status
// type:
// 0x00 - unknown
// 0x01 - data
// 0x02 - audio
// 0xff - no cdrom
// status:
// 0x00 - unknown
// 0x02 - error
// 0x08 - seek error
// 0x10 - shell open
// 0x20 - reading
// 0x40 - seeking
// 0x80 - playing
// time:
// byte 0 - minute
// byte 1 - second
// byte 2 - frame
//
// Note: only a few states are supported by my plugin right now:
// If CDDA is playing, my plugin will set Type=2, Status=0x80,
// and it will return the playing time (BCD coded). If no CDDA
// is playing, the Type will be 1. That's all right now, since I 
// can't ask the drive for more infos without confusing my read 
// caching funcs.
//
////////////////////////////////////////////////////////////////////////

struct CdrStat 
{
 unsigned long Type;
 unsigned long Status;
 unsigned char Time[3]; // current playing time
};

long CALLBACK CDRgetStatus(struct CdrStat *stat) 
{
 return -1;
}
